/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/list.h>

#include <dispatch.h>
#include <log.h>

#ifdef __EXTRA_CHECK__
static void show_tasks(void)
{
  struct task_struct *t;
  int active;

  active = 0;

  for_each_task(t) {
    cbs_dbg("T%d:\t", t->pid);

    if (t->private_data) {
      cbs_dbg("Generic Scheduler");
    } else {
      cbs_dbg("non Generic Scheduler");
    }
    if (t->rt_priority == DISPATCH_PRIORITY) {
      if (t->state != TASK_RUNNING) {
        sched_error("Scheduling ERROR: Task %d has rt_priority = %ld and state = %ld\n",
			t->pid, t->rt_priority, t->state);
      }
	active++;
    }

    cbs_dbg("priority: %ld  policy: %ld  state: %ld\n",
		    t->rt_priority, t->policy, t->state);
  }
  if (active > 1) {
    sched_error("Scheduling ERROR: Too many active tasks!!! But doing nothing...\n");
  }
  
  if (active == 0) {
    sched_print("No active tasks?\n");
  }
}
#endif

void stop(struct task_struct *c)
{
  unsigned long flags;

  if (c == NULL) {
    sched_error("CBS ERROR: Scheduling from idle!!!\n");

    return;
  }

  read_lock_irqsave(&tasklist_lock, flags);
  if (c->state == TASK_RUNNING) {
    set_task_state(c, GENSCHED_DISABLED);
  }
  set_tsk_need_resched(c);
  read_unlock_irqrestore(&tasklist_lock, flags);
}

void dispatch(struct task_struct *t, int cpu)
{
  unsigned long flags;

  if (t != NULL) {
    read_lock_irqsave(&tasklist_lock, flags);
    if (t->state == GENSCHED_DISABLED) {
      if (!t->array) {
        set_task_state(t, TASK_STOPPED); /* To fake wake_up_process() */
        wake_up_process(t);
      } else {
        sched_extra_warning("DISPATCH ERROR: dispatching TASK_DISABLED task with array != NULL!\n");
        set_task_state(t, TASK_RUNNING);
      }
    }
    set_tsk_need_resched(t);
    read_unlock_irqrestore(&tasklist_lock, flags);
  }
}

void gensched_idle_cpu(int cpu)
{
  dispatch(NULL, cpu);
}
