/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#ifndef __CBS_H__

#include <linux/kernel.h>
#include <linux/sched.h>

#include <timers.h>

#if 0
#if defined(MODVERSIONS) && !defined(__GENKSYMS__)
#include <hooks.ver>
#endif
#endif


/* For GRUB:
 * Ui = max_budget/period;
 * Vi = deadline - c*period/max_budget;
 */

struct cbs_struct {
  unsigned long long int deadline;
  unsigned long int period;
  unsigned long int max_budget;
  unsigned long int period_clock;
  unsigned long int max_budget_clock;
  long long int c;
  int new_task;
#ifdef __MULTITASK__
  struct list_head tasks;
  int activations;
#else
  struct task_struct *task;
#endif
#ifdef __CBS_GET_ERROR__
  struct list_head act_times;
  int allocated_act;
#endif
  struct list_head rlist;
#ifdef __GRUB__
  int state; /* 1 ActiveContending - 2 ActiveNonContending - 3 Inactive */
  struct t_data_struct inactive; /* timer to become inactive */
#endif
#ifdef __SLEEP__
  struct t_data_struct reactive; /* timer to become active after being blocked */
#endif
};

#ifdef __GRUB__
#define fix_point 1000 /* Used for fixed point variables ( U ) */
#endif

struct task_list {
  struct task_struct *task;
  struct list_head others;
};

struct arrival_list {
  unsigned long long int r;
  struct list_head others;
};

#define CBS_SIGNATURE 0xCB
#define E_SIGNATURE 0xCE
#define SCHED_CBS 666

struct cbs_param {
  unsigned long int signature;
  int period;
  int max_budget;
};

struct cbs_error {
  unsigned long int signature;
  unsigned long int e;
  unsigned long int period;
  unsigned long int max_budget;
};

int cbs_activate(struct cbs_struct *t, unsigned long long r);
int cbs_deactivate(struct cbs_struct *t, unsigned long long int time);
struct cbs_struct *cbs_create(struct task_struct *task, int period, int max_budget);
#ifdef __MULTITASK__
struct cbs_struct *cbs_add(struct task_struct *task, struct cbs_struct *cbs);
int cbs_remove(struct task_struct *task, struct cbs_struct *cbs);
int cbs_reactivate(struct cbs_struct *t, unsigned long long r);
#endif
void cbs_destroy(struct cbs_struct *t);
void cbs_init(void);
void cbs_deinit(void);
int cbs_get_params(struct task_struct *p, unsigned long int *qs, unsigned long int *ts);
int cbs_set_params(struct task_struct *p, unsigned long int qs, unsigned long int ts);
#endif /* __CBS_H__ */
