/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#ifndef __CBS_H__
#define __CBS_H__

#include <linux/kernel.h>
#include <linux/sched.h>

#include <timers.h>

#define STATE_NONE 0

#define RSV_ACTIVE 1	/* Server is in EDF queue. Same as GRUB_CONTENDING */
#define RSV_INACTIVE 3	/* Server has no tasks to execute */
#define RSV_DEPLETED 4	/* Server is depleted (hard rsv enforcement) */

#define GRUB_CONTENDING 1
#define GRUB_NON_CONTENDING 2
#define GRUB_INACTIVE 3
/* For GRUB:
 * Ui = max_budget/period;
 * Vi = deadline - c*period/max_budget;
 */

struct cbs_struct {
  unsigned long long int deadline;
  unsigned long int period;
  unsigned long int max_budget;
  unsigned long int period_clock;
  unsigned long int max_budget_clock;
  long long int c;
  int new_task;
#ifdef __MULTITASK__
  struct list_head tasks;
  int activations;
#else
  struct task_struct *task;
#endif
  unsigned long long int arr_time;
  struct list_head rlist;
#if defined(__GRUB__) || defined(__HARD_RSV__)
  /* 1 ActiveContending - 2 ActiveNonContending - 3 Inactive */
  int server_state;
#endif
#ifdef __GRUB__
  struct t_data_struct inactive; /* timer to become inactive */
#endif
#ifdef __HARD_RSV__
  struct t_data_struct reactivate; /* timer to become active after being blocked */
#endif
#ifdef __SRV_ID__
  int id;
#define srv_id(s) (s)->id
#define set_srv_id(s, n) (s)->id = (n)
#else
#define srv_id(s)
#define set_srv_id(s, n)
#endif
};

#ifdef __HARD_RSV__
#define rsv_reactivation_stop(t) sched_stop_timer(&(t)->reactivate)
#else
#define rsv_reactivation_stop(t)
#endif
#ifdef __GRUB__
#define fix_point 1000 /* Used for fixed point variables ( U ) */
#endif
#if defined(__GRUB__) || defined(__HARD_RSV__)
#define rsv_set_state(t, s) (t)->server_state = (s)
#define rsv_state(t) (t)->server_state
#else
#define rsv_set_state(t, s)
#endif
struct task_list {
  struct task_struct *task;
  struct list_head others;
};

int cbs_activate(struct cbs_struct *t, unsigned long long r);
int cbs_deactivate(struct cbs_struct *t, unsigned long long int time);
struct cbs_struct *cbs_create(struct task_struct *task, int period, int max_budget);
#ifdef __MULTITASK__
struct cbs_struct *cbs_add(struct task_struct *task, struct cbs_struct *cbs);
int cbs_remove(struct task_struct *task, struct cbs_struct *cbs);
int cbs_reactivate(struct cbs_struct *t, unsigned long long r);
#endif
void cbs_destroy(struct cbs_struct *t, unsigned long long int time);
void cbs_init(void);
void cbs_deinit(void);
#endif /* __CBS_H__ */
