/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#ifndef __TIME_H__
#define __TIME_H__

#include <linux/module.h>
#ifndef CONFIG_GENERIC_SCHEDULER
#error The Generic Scheduler is not enabled!!!
#endif

extern unsigned long cpu_khz;
extern unsigned fast_gettimeoffset_quotient;

static inline unsigned long long sched_read_clock(void)
{
  unsigned long long res;
  __asm__ __volatile__( "rdtsc" : "=A" (res));

  return res;
}

/* static inline long long llimd(long long ll, int mult, int div); */

/* fast_gettimeoffset_quotient = 2^32 * (1 / (TSC clocks per usec)) */
static inline unsigned long long int us2clock(unsigned long u)
{
  return llimd(u, cpu_khz, 1000);
}

static inline unsigned long int clock2ms(unsigned long long c)
{
  return (unsigned long int)(llimd(c, fast_gettimeoffset_quotient, 1) >> 32) / 1000;
}

static inline unsigned long long int clock2us(unsigned long long c)
{
  return (unsigned long long int)(llimd(c, fast_gettimeoffset_quotient, 1) >> 32);
}

static inline unsigned long int clock2jiffies(unsigned long long c)
{
  unsigned long long int tmp;
  
  tmp = llimd(c, fast_gettimeoffset_quotient, 1) >> 32;

  return llimd(tmp, HZ, 1000000);
}

static inline unsigned long int clock2subjiffies(unsigned long long c)
{
  unsigned long long int tmp, tmp1, tmp2;
  
  tmp = llimd(c, fast_gettimeoffset_quotient, 1) >> 32;
  tmp1 = llimd(tmp, HZ, 1000000);

  tmp2 = llimd(tmp1, 1000000, HZ);
  return c - llimd(tmp2, cpu_khz, 1000);
}
#endif /* __TIME_H__ */
