/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <timers.h>
#include <muldiv.h>
#include <time.h>

#include <log.h>

#define TIMER_THRESHOLD 166000

#ifdef __TIMERS_CHECK__
static int timer_id = 0;
#endif

extern int debug_code;
extern spinlock_t generic_scheduler_lock;

void sched_timeout(unsigned long arg)
{
  struct t_data_struct *tdata;
  unsigned long long int time;
#ifdef __PROFILE__
  long long int t;
#endif
  unsigned long flags;
  
  if (debug_code != 0) {
    printk("ERROR!!!! debug_code = %d != 0\n",debug_code);
  }
  time = sched_read_clock();
  tdata = (struct t_data_struct *) arg;
#ifdef __TIMERS_CHECK__
  cbs_dbg("Timer %d fires!!!\n", tdata->id);
  tdata->id = -1;
#endif
#ifdef __PROFILE__
  t = time - (tdata->requested_interval + tdata->start_time);
/*  if (t > TIMER_THRESHOLD) */{
    sched_print("Have to sleep for %Lu, slept %Lu.    Skew: %Ld\n",
		  tdata->requested_interval, time - tdata->start_time, t);
  }
#endif
  spin_lock_irqsave(&generic_scheduler_lock, flags);
  if (tdata->armed == 1) {
    tdata->armed = 0;
    tdata->callback(time, tdata->param);
  }
  spin_unlock_irqrestore(&generic_scheduler_lock, flags);
}

void sched_init_timer(struct t_data_struct *tdata)
{
  init_timer(&tdata->timer);
  tdata->armed = 0;
#ifdef __TIMERS_CHECK__
  tdata->id = -1;
#endif
}

void sched_stop_timer(struct t_data_struct *tdata)
{
  if (tdata->armed == 0) {
    return;
  }
  tdata->armed = 0;
  del_timer_sync(&(tdata->timer));
#ifdef __TIMERS_CHECK__
  cbs_dbg("Timer %d canceled\n", tdata->id);
  tdata->id = -1;
#endif
}

void sched_set_timer(unsigned long long int t, struct t_data_struct *tdata)
{
#ifdef __TIMERS_CHECK__
  if (tdata->id != -1) {
    sched_print("Hmmm... Strange timer...(id:%d)\n",tdata->id);
    printk("Hmmm... Strange timer...(id:%d) --- Current id: %d\n",tdata->id, timer_id);

    return;
  }
  tdata->id = timer_id++;
#endif

  if (tdata->armed == 1) {
    BUG();
  }
  tdata->armed = 1;
  tdata->timer.expires = clock2jiffies(t) + jiffies;
  if (tdata->timer.expires == jiffies) {
    tdata->timer.expires++;
  }
  tdata->timer.data = (unsigned long)tdata;
  tdata->timer.function = sched_timeout;
#ifdef __PROFILE__
  tdata->start_time = sched_read_clock();
  tdata->requested_interval = t;
#endif
  add_timer(&(tdata->timer));
#ifdef __TIMERS_CHECK__
  cbs_dbg("Timer %d set at jiffy %lu\n", tdata->id, tdata->timer.expires);
#else
  cbs_dbg("Timer set at jiffy %lu\n", tdata->timer.expires);
#endif
}
