/*
 * Copyright (C) 2002 Luca Abeni
 * This is Free Software; see GPL.txt for details
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>

#include <muldiv.h>
#include <time.h>
#include <trace.h>
#include <log.h>

#include <logger.h>

#define TASK_ARRIVAL 0
#define TASK_SCHEDULE 1
#define TASK_DESCHEDULE 2
#define TASK_END 3
#define TASK_DLINEPOST 4
#define TASK_DLINESET 5
#define TASK_WAIT 6
#define TASK_SIGNAL 7
#define TASK_IDLE 8
#define TASK_NAME 9

static int exec;
static unsigned long long int start_time;

#define __BINARY_TRACE__

#ifdef __BINARY_TRACE__
static void trace_log_int(int v)
{
  int sw;

  sw = htonl(v);
  log_write(1, (char *)&sw, sizeof(int));
}
#else
static void trace_log_int(int v)
{
  int sw;

  log_printf(1, "%d ", v);
}
#endif

static void trace_common(int type, int time, int task)
{
  trace_log_int(type);
  trace_log_int(time);
  trace_log_int((int)task);
}

void trc_dispatch(int pid, unsigned long long int time)
{
  int utime;

  utime = clock2ms(time - start_time);
  if (exec != 0) {
    trace_common(TASK_DESCHEDULE, utime, exec);
  }
  exec = pid;
  if (pid != 0) {
    trace_common(TASK_SCHEDULE, utime, pid);
  }
}

void trc_activation(int pid, unsigned long long int time)
{
  int utime;

  utime = clock2ms(time - start_time);
  trace_common(TASK_ARRIVAL, utime, pid);
}

void trc_deactivation(int pid, unsigned long long int time)
{
  int utime;

  utime = clock2ms(time - start_time);
  trace_common(TASK_END, utime, pid);
}

void trc_creation(int pid, unsigned long long int time)
{
  int utime;
  int len;
  char name[16];

  utime = clock2ms(time - start_time);
  trace_common(TASK_NAME, utime, pid);
  sprintf(name, "task%d", (int)pid);
  len = strlen(name);
  trace_log_int(len);
  log_write(1, name, len);
}

void trc_destruction(int pid, unsigned long long int time)
{
  /* Nothing to do, here... */
}

void trc_dl_postpone(int pid, unsigned long long int time,
		unsigned long long int old_dl, unsigned long long int new_dl)
{
  int utime;

  utime = clock2ms(time - start_time);
  trace_common(TASK_DLINEPOST, utime, pid);
  trace_log_int(clock2ms(old_dl - start_time));
  trace_log_int(clock2ms(new_dl - start_time));
}

void trc_dl_assign(int pid, unsigned long long int time,
		unsigned long long int new_dl)
{
  int utime;

  utime = clock2ms(time - start_time);
  trace_common(TASK_DLINESET, utime, pid);
  trace_log_int(clock2ms(new_dl - start_time));
}

void trc_start(unsigned long long int time)
{
  start_time = time;
}
